package com.datarp.calc;

import java.util.*;

public class CalculatorTest {

	public static int NUM_TESTS = 10000;
	public static int PRECISION = 9;
	public static double SEARCH = 0.0001f;


	public static void main(String args[]) {
		additionTest();
		subtractionTest();
		multiplicationTest();
		divisionTest();
	}

	public static void additionTest() {
		for (int i=0; i<NUM_TESTS; i++) {
			String a = randomNumber((char)PRECISION);
			String b = randomNumber((char)PRECISION);
			String c = randomNumber((char)PRECISION);
			String expr = a+" + "+b+" + "+c;
			String trial = new Calculator(expr,PRECISION).evaluate();
			double answer = decode(a)+decode(b)+decode(c);
			double trialf = decode(trial);
			double percent = Math.abs((answer-trialf)/answer*100.0);
			if ( percent > SEARCH ) {
				System.out.println("Testing   : "+expr);
				System.out.println("Calculator: "+trial);
				System.out.println("Java VM   : "+answer);
				System.out.println("% diff    : "+percent);
				System.out.println("");
			}
		}
	}

	public static void subtractionTest() {
		for (int i=0; i<NUM_TESTS; i++) {
			String a = randomNumber((char)PRECISION);
			String b = randomNumber((char)PRECISION);
			String c = randomNumber((char)PRECISION);
			String expr = a+" - "+b+" - "+c;
			String trial = new Calculator(expr,PRECISION).evaluate();
			double answer = decode(a)-decode(b)-decode(c);
			double trialf = decode(trial);
			double percent = Math.abs((answer-trialf)/answer*100.0);
			if ( percent > SEARCH ) {
				System.out.println("Testing   : "+expr);
				System.out.println("Calculator: "+trial);
				System.out.println("Java VM   : "+answer);
				System.out.println("% diff    : "+percent);
				System.out.println("");
			}
		}
	}

	public static void multiplicationTest() {
		for (int i=0; i<NUM_TESTS; i++) {
			String a = randomNumber((char)PRECISION);
			String b = randomNumber((char)PRECISION);
			String c = randomNumber((char)PRECISION);
			String expr = a+" * "+b+" * "+c;
			String trial = new Calculator(expr,PRECISION).evaluate();
			double answer = decode(a)*decode(b)*decode(c);
			double trialf = decode(trial);
			double percent = Math.abs((answer-trialf)/answer*100.0);
			if ( percent > SEARCH ) {
				System.out.println("Testing   : "+expr);
				System.out.println("Calculator: "+trial);
				System.out.println("Java VM   : "+answer);
				System.out.println("% diff    : "+percent);
				System.out.println("");
			}
		}
	}

	public static void divisionTest() {
		for (int i=0; i<NUM_TESTS; i++) {
			String a = randomNumber((char)PRECISION);
			String b = randomNumber((char)PRECISION);
			String c = randomNumber((char)PRECISION);
			String expr = a+" / "+b+" / "+c;
			String trial = new Calculator(expr,PRECISION).evaluate();
			double answer = decode(a)/decode(b)/decode(c);
			double trialf = decode(trial);
			double percent = Math.abs((answer-trialf)/answer*100.0);
			if ( percent > SEARCH ) {
				System.out.println("Testing   : "+expr);
				System.out.println("Calculator: "+trial);
				System.out.println("Java VM   : "+answer);
				System.out.println("% diff    : "+percent);
				System.out.println("");
			}
		}
	}

	private static Random random = new Random(System.currentTimeMillis());

	public static char randomUnsigned(char max) {
		return (char)(((char)random.nextInt())%(max+1));
	}

	public static char randomNumeral() {
		return (char)('0' + randomUnsigned((char)9));
	}

	public static String randomNumber(char precision) {
		StringBuffer sb = new StringBuffer();
		for ( int i=0; i<precision; i++ ) sb.append(randomNumeral());
		sb.insert( randomUnsigned(precision), '.' );
		sb.append( 'e' );
		if ( randomUnsigned((char)9)>4 ) sb.append( '-' );
		sb.append((int)randomUnsigned((char)50));
		if ( randomUnsigned((char)9)>4 ) sb.insert( 0, '-' );
		return sb.toString();
	}

	public static double decode(String f) {
		return new Double(f).doubleValue();
	}

}

